@@ -50,9 +50,12 @@ def _assert_tensor_graph_name(node: ast.GraphNode, expects,
     return call
 
 
-def _merge_shapes(last_outputs, new_inputs):
+def merge_shapes(last_outputs, new_inputs):
     for name in last_outputs.keys():
         last_output = last_outputs[name]
+
+        if name not in new_inputs.keys():
+            continue
         new_input = new_inputs[name]
 
         # dynamic size
@@ -72,10 +75,22 @@ def _merge_shapes(last_outputs, new_inputs):
                     new_dim.ty = ast.NodeLetType.INT
                     new_dim.value = last_dim
                 # test value
-                elif ast.is_estimable(last_dim) and ast.is_estimable(new_dim):
+                else:
                     last_dim = build_variable(last_dim)
                     new_dim = build_variable(new_dim)
-                    assert last_dim == new_dim, f'Expected {new_dim}, Given {last_dim}'
+                    _assert_equal(last_dim, new_dim)
+            else:
+                # link
+                if isinstance(new_dim, ast.Variable) and new_dim.ty == ast.NodeLetType.DIM:
+                    new_dim.value = last_dim
+
+
+def _assert_equal(last, new):
+    if ast.is_hint(last) or ast.is_hint(new):
+        # TODO: FUTURE: implement comparing hinted values
+        print('warning: comparing hinted values is not supported yet!')
+    else:
+        assert last == new, f'Expected {last}, Given {new}'
 
 
 class NodeIR:
@@ -237,7 +252,7 @@ class NodeContainer:
                                     for k, x in inputs.items()}
                     new_inputs = callee.get_input_shapes()
 
-                    _merge_shapes(last_outputs, new_inputs)
+                    merge_shapes(last_outputs, new_inputs)
 
                     # identity
                     new_outputs = callee.get_output_shapes()
@@ -254,7 +269,7 @@ class NodeContainer:
             # Step 6. merge dedicated shapes
             if node.shapes:
                 outputs = self.get_output_shapes()
-                _merge_shapes(node.shapes, outputs)
+                merge_shapes(node.shapes, outputs)
 
             # Step 7. store id
             self.last_id = node.id
@@ -328,11 +343,7 @@ class NodeContainer:
         for name in input_keys:
             input = build_variable(inputs[name].product())
             output = build_variable(outputs[name].product())
-            if ast.is_hint(input) or ast.is_hint(output):
-                # TODO: FUTURE: implement comparing hinted values
-                print('warning: comparing hinted values is not supported yet!')
-            else:
-                assert input == output, f'Expected product {input}, Given {output}'
+            _assert_equal(input, output)
 
         # Step 4. apply to tensor graph
         id = self.last_id + 1
@@ -371,22 +382,22 @@ class NodeContainer:
         tensor_base[axis] = None
 
         for shape in inputs[1:]:
-            assert len(
-                shape) == tensor_dims, f'Expected <{tensor_dims}> dims Tensor, Given <{len(shape)}>{shape}'
+            assert len(shape) == tensor_dims, \
+                f'Expected <{tensor_dims}> dims Tensor, Given <{len(shape)}>{shape}'
             for d0, d1 in zip(tensor_base, shape.dims):
                 if d0 is None:
                     target_dim.dims.append(d1)
                     continue
                 d0 = build_variable(d0)
                 d1 = build_variable(d1)
-                if ast.is_hint(d0) or ast.is_hint(d1):
-                    # TODO: FUTURE: implement comparing hinted values
-                    print('warning: comparing hinted values is not supported yet!')
-                assert d0 == d1, f'Expected {d0}, Given {d1}'
+                _assert_equal(d0, d1)
+
+        outputs = tensor_base
+        outputs[axis] = target_dim.add()
 
         # Step 4. get the IO
         inputs = {'x': inputs}
-        outputs = {'x': ast.Shape([target_dim.add()])}
+        outputs = {'x': ast.Shape(outputs)}
 
         # Step 5. apply to tensor graph
         id = self.last_id + 1
@@ -488,17 +499,17 @@ class ExecIR(EnvVars):
             for var in self._graph.get_variables().values():
                 value = getattr(self, var.name.replace(' ', '_'))
                 if var.ty.is_node():
-                    ir = root.node_root.get(value)
-                    var.value = node = ir.build(root.node_root)
+                    var.value = root.node_root.get(value)
                 else:
                     var.value = build_variable(value)
 
-        # Step 3. get variables
-        kwargs = {k: v.value for k, v in self._graph.get_variables().items()}
+            # Step 3. get variables
+            kwargs = {k: v.value for k,
+                      v in self._graph.get_variables().items()}
 
-        # Step 4. build to extern exec node
-        node_cls = root.node_root.get_extern(self._name)
-        return node_cls(env=root.env, **kwargs)
+            # Step 4. build to extern exec node
+            node_cls = root.node_root.get_extern(self._name)
+            return node_cls(env=root.env, root=root.node_root, **kwargs)
 
     def _use_exec(self, root):
         class ReadOnlyExec:
