@@ -1,5 +1,7 @@
 import inflection
+from copy import copy
 
+from ..ast import clone_value
 from .variable import build
 
 
@@ -17,6 +19,14 @@ class TensorGraph:
         self.output_shapes = output_shapes or {
             k: None for k in self.output.keys()}
 
+    def clone(self, root, variables):
+        kwargs = clone_value(self.kwargs, variables)
+        input_shapes = clone_value(self.input_shapes, variables)
+        output_shapes = clone_value(self.output_shapes, variables)
+
+        return TensorGraph(self.id, self.name, kwargs, copy(self.input), copy(self.output),
+                           input_shapes, output_shapes)
+
     def get_input_shapes(self):
         return self.input_shapes
 
@@ -28,7 +38,7 @@ class TensorGraph:
                   for k, v in self.kwargs.items()}
 
         node_cls = root.get_extern(self.name)
-        return node_cls(input=self.input, output=self.output, **kwargs)
+        return node_cls(exec=root.exec, input=self.input, output=self.output, **kwargs)
 
     def __repr__(self):
         kwargs = '\n\t[args]\n' + \
