@@ -5,22 +5,22 @@ import click
 
 from ..builder import NodeRoot
 from ..std import trim_name
-
-DATA_DIR = 'data'
-MODELS_DIR = 'models'
-NODES_DIR = 'nodes'
-NODES_USER_DIR = '__user__'
+from ..util.dirs import *
 
 
 class ExecRoot:
     def __init__(self, env):
         super().__init__()
         self.env = env
-        self.inner = NodeRoot()
+        self.node_root = NodeRoot()
+        self.node_root.env = env
 
         self._create_root_dir()
         self._load_local_nodes()
 
+    def get(self, name):
+        return self.node_root.get(name)
+
     @property
     def root_dir(self):
         return self.env.root
@@ -44,6 +44,11 @@ class ExecRoot:
 
     def _load_local_nodes(self):
         path = os.path.join(self.root_dir, 'nodes')
-        for p in Path(path).glob('*.n3'):
+        for p in Path(path).glob('**/*.n3'):
             name = trim_name(p.name)
-            self.inner.add_source_path(name, str(p))
+            self.node_root.add_source_path(name, str(p))
+
+            extern_name = p.name[:-3].replace("-", "_")
+            extern_path = os.path.join(p.parent, f'{extern_name}.py')
+            if os.path.exists(extern_path):
+                self.node_root.add_extern(name, extern_path)
