@@ -7,6 +7,16 @@ from lark.indenter import Indenter
 from .. import ast
 
 
+def try_to_dict(data, f):
+    result = {}
+    for entry in data:
+        key, value = f(entry)
+        if key in result.keys():
+            raise Exception(f'duplicated name: {key}')
+        result[key] = value
+    return result
+
+
 class _Indenter(Indenter):
     NL_type = '_NEWLINE'
     OPEN_PAREN_types = ['LPAR', 'LSQB', 'LBRACE']
@@ -64,20 +74,16 @@ class _Transformer(Transformer):
         return ast.Node(str(args[0]), *args[1:])
 
     def node_suite_let(self, args):
-        # TODO: check duplicated names
-        return {a.name: a for a in args}
+        return try_to_dict(args, lambda a: (a.name, a))
 
     def node_suite_with(self, args):
-        # TODO: check duplicated names
-        return {a.name: a for a in args}
+        return try_to_dict(args, lambda a: (a.name, a))
 
     def node_suite_children(self, args):
-        # TODO: check duplicated names
-        return {a.name: a for a in args}
+        return try_to_dict(args, lambda a: (a.name, a))
 
     def node_suite_graph(self, args):
-        # TODO: check duplicated names
-        return {a.id: a for a in args}
+        return try_to_dict(args, lambda a: (a.id, a))
 
     def nodelet(self, args):
         name = str(args[1])
@@ -116,8 +122,7 @@ class _Transformer(Transformer):
         return ast.NodeCall(str(args[0]), args[1], args[2])
 
     def graph_inputs_dict(self, args):
-        # TODO: check duplicated names
-        return {a[0]: a[1] for a in args}
+        return try_to_dict(args, lambda a: a)
 
     def graph_inputs_list(self, args):
         return args
@@ -126,15 +131,13 @@ class _Transformer(Transformer):
         return str(args[0]), args[1]
 
     def graph_args(self, args):
-        # TODO: check duplicated names
-        return {a.name: a for a in args}
+        return try_to_dict(args, lambda a: (a.name, a))
 
     def graph_arg(self, args):
         return ast.NodeArg(str(args[0]), args[1])
 
     def graph_shapes(self, args):
-        # TODO: check duplicated names
-        return {a.name: a for a in args}
+        return try_to_dict(args, lambda a: (a.name, a))
 
     def graph_shape_kw(self, args):
         args[1].name = str(args[0])
@@ -156,8 +159,7 @@ class _Transformer(Transformer):
         return ast.With(str(args[0]), args[1])
 
     def with_suite(self, args):
-        # TODO: check duplicated names
-        return {a.name: a for a in args}
+        return try_to_dict(args, lambda a: (a.name, a))
 
     def withset(self, args):
         return ast.WithSet(args[0], args[1])
