@@ -13,7 +13,12 @@ def _read_file(f):
 
 root_dir = os.path.dirname(__file__)
 
-externs = {trim_name(f.name): str(f)
-           for f in Path(root_dir).rglob(r'[A-Za-z]*.py')}
-sources = {trim_name(f.name): _read_file(f)
-           for f in Path(root_dir).rglob(r'*.n3')}
+
+def get_externs():
+    return {trim_name(f.name): str(f)
+            for f in Path(root_dir).rglob(r'[A-Za-z]*.py')}
+
+
+def get_sources():
+    return {trim_name(f.name): str(f)
+            for f in Path(root_dir).rglob(r'*.n3')}
